@@ -73,7 +73,16 @@ public:
 protected:
 
 	void updatePreset(bool bDirty);
+	void updateParam(samplv1::ParamIndex index);
+	void updateParams();
+
 	void updateSample();
+
+	void updateOffsetRange();
+	void updateLoopRange();
+	void updateLoopFade();
+	void updateLoopZero();
+
 	void updateTuning();
 
 private:
