@@ -1,7 +1,7 @@
 // samplv1_lv2.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2020, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -54,6 +54,13 @@
 #define LV2_STATE__StateChanged LV2_STATE_PREFIX "StateChanged"
 #endif
 
+#ifndef LV2_ATOM__PortEvent
+#define LV2_ATOM__PortEvent LV2_ATOM_PREFIX "PortEvent"
+#endif
+#ifndef LV2_ATOM__portTuple
+#define LV2_ATOM__portTuple LV2_ATOM_PREFIX "portTuple"
+#endif
+
 #include <stdlib.h>
 #include <math.h>
 
@@ -69,7 +76,10 @@
 // atom-like message used internally with worker/schedule
 typedef struct {
 	LV2_Atom atom;
-	const char *sample_path;
+	union {
+		uint32_t    key;
+		const char *path;
+	} data;
 } samplv1_lv2_worker_message;
 
 
@@ -148,6 +158,12 @@ samplv1_lv2::samplv1_lv2 (
 					m_urid_map->handle, LV2_ATOM__Bool);
 				m_urids.atom_Path = m_urid_map->map(
 					m_urid_map->handle, LV2_ATOM__Path);
+			#ifdef CONFIG_LV2_PORT_EVENT
+				m_urids.atom_PortEvent = m_urid_map->map(
+					m_urid_map->handle, LV2_ATOM__PortEvent);
+				m_urids.atom_portTuple = m_urid_map->map(
+					m_urid_map->handle, LV2_ATOM__portTuple);
+			#endif
 				m_urids.time_Position = m_urid_map->map(
 					m_urid_map->handle, LV2_TIME__Position);
 				m_urids.time_beatsPerMinute = m_urid_map->map(
@@ -171,10 +187,6 @@ samplv1_lv2::samplv1_lv2 (
 					m_urid_map->handle, LV2_PATCH__Get);
 				m_urids.patch_Set = m_urid_map->map(
 					m_urid_map->handle, LV2_PATCH__Set);
-				m_urids.patch_Put = m_urid_map->map(
-					m_urid_map->handle, LV2_PATCH__Put);
-				m_urids.patch_body = m_urid_map->map(
-					m_urid_map->handle, LV2_PATCH__body);
 				m_urids.patch_property = m_urid_map->map(
 					m_urid_map->handle, LV2_PATCH__property);
 				m_urids.patch_value = m_urid_map->map(
@@ -330,8 +342,8 @@ void samplv1_lv2::run ( uint32_t nframes )
 							if (m_schedule) {
 								samplv1_lv2_worker_message mesg;
 								mesg.atom.type = key;
-								mesg.atom.size = sizeof(mesg.sample_path);
-								mesg.sample_path
+								mesg.atom.size = sizeof(mesg.data.path);
+								mesg.data.path
 									= (const char *) LV2_ATOM_BODY_CONST(value);
 								// schedule loading new sample
 								m_schedule->schedule_work(
@@ -350,7 +362,7 @@ void samplv1_lv2::run ( uint32_t nframes )
 									= *(uint32_t *) LV2_ATOM_BODY_CONST(value);
 								const uint32_t offset_end
 									= pSample->offsetEnd();
-								setOffsetRange(offset_start, offset_end);
+								setOffsetRange(offset_start, offset_end, true);
 							}
 						}
 						else
@@ -365,7 +377,7 @@ void samplv1_lv2::run ( uint32_t nframes )
 									= pSample->offsetStart();
 								const uint32_t offset_end
 									= *(uint32_t *) LV2_ATOM_BODY_CONST(value);
-								setOffsetRange(offset_start, offset_end);
+								setOffsetRange(offset_start, offset_end, true);
 							}
 						}
 						else
@@ -380,7 +392,7 @@ void samplv1_lv2::run ( uint32_t nframes )
 									= *(uint32_t *) LV2_ATOM_BODY_CONST(value);
 								const uint32_t loop_end
 									= pSample->loopEnd();
-								setLoopRange(loop_start, loop_end);
+								setLoopRange(loop_start, loop_end, true);
 							}
 						}
 						else
@@ -395,7 +407,7 @@ void samplv1_lv2::run ( uint32_t nframes )
 									= pSample->loopStart();
 								const uint32_t loop_end
 									= *(uint32_t *) LV2_ATOM_BODY_CONST(value);
-								setLoopRange(loop_start, loop_end);
+								setLoopRange(loop_start, loop_end, true);
 							}
 						}
 						else
@@ -406,7 +418,7 @@ void samplv1_lv2::run ( uint32_t nframes )
 							) && type == m_urids.atom_Int) {
 							const uint32_t loop_fade
 								= *(uint32_t *) LV2_ATOM_BODY_CONST(value);
-							setLoopFade(loop_fade);
+							setLoopFade(loop_fade, true);
 						}
 						else
 						if ((key == m_urids.p107_loop_zero
@@ -420,7 +432,7 @@ void samplv1_lv2::run ( uint32_t nframes )
 							)) {
 							const uint32_t loop_zero
 								= *(uint32_t *) LV2_ATOM_BODY_CONST(value);
-							setLoopZero(loop_zero > 0);
+							setLoopZero(loop_zero > 0, true);
 						}
 						else
 						if (key == m_urids.p201_tuning_enabled
@@ -466,8 +478,14 @@ void samplv1_lv2::run ( uint32_t nframes )
 				}
 				else
 				if (object->body.otype == m_urids.patch_Get) {
-					// put all property values (probably to UI)
-					patch_put(ndelta);
+					// get one or all property values (probably to UI)...
+					const LV2_Atom_URID *prop = nullptr;
+					lv2_atom_object_get(object,
+						m_urids.patch_property, (const LV2_Atom *) &prop, 0);
+					if (prop && prop->atom.type == m_forge.URID)
+						patch_get(prop->body);
+					else
+						patch_get(0); // all
 				}
 			#endif	// CONFIG_LV2_PATCH
 			}
@@ -515,7 +533,7 @@ void samplv1_lv2::qapp_instantiate (void)
 {
 	if (qApp == nullptr && g_qapp_instance == nullptr) {
 		static int s_argc = 1;
-		static const char *s_argv[] = { __func__, nullptr };
+		static const char *s_argv[] = { SAMPLV1_TITLE, nullptr };
 		g_qapp_instance = new QApplication(s_argc, (char **) s_argv);
 	}
 
@@ -969,20 +987,103 @@ void samplv1_lv2::updatePreset ( bool /*bDirty*/ )
 		samplv1_lv2_worker_message mesg;
 		mesg.atom.type = m_urids.state_StateChanged;
 		mesg.atom.size = 0; // nothing else matters.
-		mesg.sample_path = nullptr;
 		m_schedule->schedule_work(
 			m_schedule->handle, sizeof(mesg), &mesg);
 	}
 }
 
 
+void samplv1_lv2::updateParam ( samplv1::ParamIndex index )
+{
+#ifdef CONFIG_LV2_PORT_EVENT
+	if (m_schedule) {
+		samplv1_lv2_worker_message mesg;
+		mesg.atom.type = m_urids.atom_PortEvent;
+		mesg.atom.size = sizeof(mesg.data.key);
+		mesg.data.key  = uint32_t(index);
+		m_schedule->schedule_work(
+			m_schedule->handle, sizeof(mesg), &mesg);
+	}
+#else
+	(void) index; // STFU dang compiler!
+#endif
+}
+
+
+void samplv1_lv2::updateParams (void)
+{
+#ifdef CONFIG_LV2_PORT_EVENT
+	if (m_schedule) {
+		samplv1_lv2_worker_message mesg;
+		mesg.atom.type = m_urids.atom_PortEvent;
+		mesg.atom.size = 0; // nothing else matters.
+		m_schedule->schedule_work(
+			m_schedule->handle, sizeof(mesg), &mesg);
+	}
+#endif
+}
+
+
 void samplv1_lv2::updateSample (void)
 {
 	if (m_schedule) {
 		samplv1_lv2_worker_message mesg;
 		mesg.atom.type = m_urids.gen1_update;
-		mesg.atom.size = sizeof(mesg.sample_path);
-		mesg.sample_path = samplv1::sampleFile();
+		mesg.atom.size = 0; // nothing else matters.
+		m_schedule->schedule_work(
+			m_schedule->handle, sizeof(mesg), &mesg);
+	}
+}
+
+
+void samplv1_lv2::updateOffsetRange (void)
+{
+	if (m_schedule) {
+		samplv1_lv2_worker_message mesg;
+		mesg.atom.type = m_urids.p102_offset_start;
+		mesg.atom.size = 0; // nothing else matters.
+		m_schedule->schedule_work(
+			m_schedule->handle, sizeof(mesg), &mesg);
+		mesg.atom.type = m_urids.p103_offset_end;
+		m_schedule->schedule_work(
+			m_schedule->handle, sizeof(mesg), &mesg);
+	}
+}
+
+
+void samplv1_lv2::updateLoopRange (void)
+{
+	if (m_schedule) {
+		samplv1_lv2_worker_message mesg;
+		mesg.atom.type = m_urids.p104_loop_start;
+		mesg.atom.size = 0; // nothing else matters.
+		m_schedule->schedule_work(
+			m_schedule->handle, sizeof(mesg), &mesg);
+		mesg.atom.type = m_urids.p105_loop_end;
+		m_schedule->schedule_work(
+			m_schedule->handle, sizeof(mesg), &mesg);
+	}
+}
+
+
+void samplv1_lv2::updateLoopFade (void)
+{
+	if (m_schedule) {
+		samplv1_lv2_worker_message mesg;
+		mesg.atom.type = m_urids.p106_loop_fade;
+		mesg.atom.size = 0; // nothing else matters.
+		m_schedule->schedule_work(
+			m_schedule->handle, sizeof(mesg), &mesg);
+	}
+}
+
+
+void samplv1_lv2::updateLoopZero (void)
+{
+	if (m_schedule) {
+		samplv1_lv2_worker_message mesg;
+		mesg.atom.type = m_urids.p107_loop_zero;
+		mesg.atom.size = 0; // nothing else matters.
 		m_schedule->schedule_work(
 			m_schedule->handle, sizeof(mesg), &mesg);
 	}
@@ -1009,27 +1110,13 @@ bool samplv1_lv2::worker_work ( const void *data, uint32_t size )
 	const samplv1_lv2_worker_message *mesg
 		= (const samplv1_lv2_worker_message *) data;
 
-	if (mesg->atom.type == m_urids.state_StateChanged)
-		return true;
+	if (mesg->atom.type == m_urids.p101_sample_file)
+		samplv1::setSampleFile(mesg->data.path);
 	else
-	if (mesg->atom.type == m_urids.gen1_update)
-		return true;
-	else
-	if (mesg->atom.type == m_urids.p101_sample_file
-	#if 0//SAMPLV1_LV2_LEGACY
-		|| mesg->atom.type == m_urids.gen1_sample
-	#endif
-		) {
-		samplv1::setSampleFile(mesg->sample_path);
-		return true;
-	}
-	else
-	if (mesg->atom.type == m_urids.tun1_update) {
+	if (mesg->atom.type == m_urids.tun1_update)
 		samplv1::resetTuning();
-		return true;
-	}
 
-	return false;
+	return true;
 }
 
 
@@ -1040,6 +1127,16 @@ bool samplv1_lv2::worker_response ( const void *data, uint32_t size )
 
 	const samplv1_lv2_worker_message *mesg
 		= (const samplv1_lv2_worker_message *) data;
+
+#ifdef CONFIG_LV2_PORT_EVENT
+	if (mesg->atom.type == m_urids.atom_PortEvent) {
+		if (mesg->atom.size > 0)
+			return port_event(samplv1::ParamIndex(mesg->data.key));
+		else
+			return port_events();
+	}
+	else
+#endif
 	if (mesg->atom.type == m_urids.state_StateChanged)
 		return state_changed();
 
@@ -1047,7 +1144,7 @@ bool samplv1_lv2::worker_response ( const void *data, uint32_t size )
 	samplv1_sched::sync_notify(this, samplv1_sched::Sample, 0);
 
 #ifdef CONFIG_LV2_PATCH
-	return patch_put(m_ndelta, mesg->atom.type);
+	return patch_get(mesg->atom.type);
 #else
 	return true;
 #endif
@@ -1068,94 +1165,156 @@ bool samplv1_lv2::state_changed (void)
 
 #ifdef CONFIG_LV2_PATCH
 
-bool samplv1_lv2::patch_put ( uint32_t ndelta, uint32_t type )
+bool samplv1_lv2::patch_set ( LV2_URID key )
 {
 	static char s_szNull[1] = {'\0'};
 
-	if (type == m_urids.patch_Put)
-		type = 0;
-
 	samplv1_sample *pSample = samplv1::sample();
 	if (pSample == nullptr)
 		return false;
 
-	lv2_atom_forge_frame_time(&m_forge, ndelta);
+	lv2_atom_forge_frame_time(&m_forge, m_ndelta);
 
 	LV2_Atom_Forge_Frame patch_frame;
-	lv2_atom_forge_object(&m_forge, &patch_frame, 0, m_urids.patch_Put);
-	lv2_atom_forge_key(&m_forge, m_urids.patch_body);
+	lv2_atom_forge_object(&m_forge, &patch_frame, 0, m_urids.patch_Set);
 
-	LV2_Atom_Forge_Frame body_frame;
-	lv2_atom_forge_object(&m_forge, &body_frame, 0, 0);
+	lv2_atom_forge_key(&m_forge, m_urids.patch_property);
+	lv2_atom_forge_urid(&m_forge, key);
 
-	if (type == 0 || type == m_urids.p101_sample_file) {
+	lv2_atom_forge_key(&m_forge, m_urids.patch_value);
+
+	if (key == m_urids.p101_sample_file) {
 		const char *pszSampleFile = pSample->filename();
 		if (pszSampleFile == nullptr)
 			pszSampleFile = s_szNull;
-		lv2_atom_forge_key(&m_forge, m_urids.p101_sample_file);
 		lv2_atom_forge_path(&m_forge, pszSampleFile, ::strlen(pszSampleFile) + 1);
 	}
-	if (type == 0 || type == m_urids.p102_offset_start) {
-		lv2_atom_forge_key(&m_forge, m_urids.p102_offset_start);
+	else
+	if (key == m_urids.p102_offset_start)
 		lv2_atom_forge_int(&m_forge, pSample->offsetStart());
-	}
-	if (type == 0 || type == m_urids.p103_offset_end) {
-		lv2_atom_forge_key(&m_forge, m_urids.p103_offset_end);
+	else
+	if (key == m_urids.p103_offset_end)
 		lv2_atom_forge_int(&m_forge, pSample->offsetEnd());
-	}
-	if (type == 0 || type == m_urids.p104_loop_start) {
-		lv2_atom_forge_key(&m_forge, m_urids.p104_loop_start);
+	else
+	if (key == m_urids.p104_loop_start)
 		lv2_atom_forge_int(&m_forge, pSample->loopStart());
-	}
-	if (type == 0 || type == m_urids.p105_loop_end) {
-		lv2_atom_forge_key(&m_forge, m_urids.p105_loop_end);
+	else
+	if (key == m_urids.p105_loop_end)
 		lv2_atom_forge_int(&m_forge, pSample->loopEnd());
-	}
-	if (type == 0 || type == m_urids.p106_loop_fade) {
-		lv2_atom_forge_key(&m_forge, m_urids.p106_loop_fade);
+	else
+	if (key == m_urids.p106_loop_fade)
 		lv2_atom_forge_int(&m_forge, pSample->loopCrossFade());
-	}
-	if (type == 0 || type == m_urids.p107_loop_zero) {
-		lv2_atom_forge_key(&m_forge, m_urids.p107_loop_zero);
+	else
+	if (key == m_urids.p107_loop_zero)
 		lv2_atom_forge_bool(&m_forge, pSample->isLoopZeroCrossing());
-	}
-
-	if (type == 0 || type == m_urids.p201_tuning_enabled) {
-		lv2_atom_forge_key(&m_forge, m_urids.p201_tuning_enabled);
+	else
+	if (key == m_urids.p201_tuning_enabled)
 		lv2_atom_forge_bool(&m_forge, samplv1::isTuningEnabled());
-	}
-	if (type == 0 || type == m_urids.p202_tuning_refPitch) {
-		lv2_atom_forge_key(&m_forge, m_urids.p202_tuning_refPitch);
+	else
+	if (key == m_urids.p202_tuning_refPitch)
 		lv2_atom_forge_float(&m_forge, samplv1::tuningRefPitch());
-	}
-	if (type == 0 || type == m_urids.p203_tuning_refNote) {
-		lv2_atom_forge_key(&m_forge, m_urids.p203_tuning_refNote);
+	else
+	if (key == m_urids.p203_tuning_refNote)
 		lv2_atom_forge_int(&m_forge, samplv1::tuningRefNote());
-	}
-	if (type == 0 || type == m_urids.p204_tuning_scaleFile) {
+	else
+	if (key == m_urids.p204_tuning_scaleFile) {
 		const char *pszScaleFile = samplv1::tuningScaleFile();
 		if (pszScaleFile == nullptr)
 			pszScaleFile = s_szNull;
-		lv2_atom_forge_key(&m_forge, m_urids.p204_tuning_scaleFile);
 		lv2_atom_forge_path(&m_forge, pszScaleFile, ::strlen(pszScaleFile) + 1);
 	}
-	if (type == 0 || type == m_urids.p205_tuning_keyMapFile) {
+	else
+	if (key == m_urids.p205_tuning_keyMapFile) {
 		const char *pszKeyMapFile = samplv1::tuningKeyMapFile();
 		if (pszKeyMapFile == nullptr)
 			pszKeyMapFile = s_szNull;
-		lv2_atom_forge_key(&m_forge, m_urids.p205_tuning_keyMapFile);
 		lv2_atom_forge_path(&m_forge, pszKeyMapFile, ::strlen(pszKeyMapFile) + 1);
 	}
 
-	lv2_atom_forge_pop(&m_forge, &body_frame);
 	lv2_atom_forge_pop(&m_forge, &patch_frame);
 
 	return true;
 }
 
+bool samplv1_lv2::patch_get ( LV2_URID key )
+{
+	if (key == 0 || key == m_urids.gen1_update) {
+		patch_set(m_urids.p101_sample_file);
+		patch_set(m_urids.p102_offset_start);
+		patch_set(m_urids.p103_offset_end);
+		patch_set(m_urids.p104_loop_start);
+		patch_set(m_urids.p105_loop_end);
+		patch_set(m_urids.p106_loop_fade);
+		patch_set(m_urids.p107_loop_zero);
+		if (key) return true;
+	}
+
+	if (key == 0 || key == m_urids.tun1_update) {
+		patch_set(m_urids.p201_tuning_enabled);
+		patch_set(m_urids.p202_tuning_refPitch);
+		patch_set(m_urids.p203_tuning_refNote);
+		patch_set(m_urids.p204_tuning_scaleFile);
+		patch_set(m_urids.p205_tuning_keyMapFile);
+		if (key) return true;
+	}
+
+	if (key) patch_set(key);
+
+	return true;
+}
+
 #endif	// CONFIG_LV2_PATCH
 
 
+#ifdef CONFIG_LV2_PORT_EVENT
+
+bool samplv1_lv2::port_event ( samplv1::ParamIndex index )
+{
+	lv2_atom_forge_frame_time(&m_forge, m_ndelta);
+
+	LV2_Atom_Forge_Frame obj_frame;
+	lv2_atom_forge_object(&m_forge, &obj_frame, 0, m_urids.atom_PortEvent);
+	lv2_atom_forge_key(&m_forge, m_urids.atom_portTuple);
+
+	LV2_Atom_Forge_Frame tup_frame;
+	lv2_atom_forge_tuple(&m_forge, &tup_frame);
+
+	lv2_atom_forge_int(&m_forge, int32_t(ParamBase + index));
+	lv2_atom_forge_float(&m_forge, samplv1::paramValue(index));
+
+	lv2_atom_forge_pop(&m_forge, &tup_frame);
+	lv2_atom_forge_pop(&m_forge, &obj_frame);
+
+	return true;
+}
+
+
+bool samplv1_lv2::port_events (void)
+{
+	lv2_atom_forge_frame_time(&m_forge, m_ndelta);
+
+	LV2_Atom_Forge_Frame obj_frame;
+	lv2_atom_forge_object(&m_forge, &obj_frame, 0, m_urids.atom_PortEvent);
+	lv2_atom_forge_key(&m_forge, m_urids.atom_portTuple);
+
+	LV2_Atom_Forge_Frame tup_frame;
+	lv2_atom_forge_tuple(&m_forge, &tup_frame);
+
+	for (uint32_t i = 0; i < samplv1::NUM_PARAMS; ++i) {
+		samplv1::ParamIndex index = samplv1::ParamIndex(i);
+		lv2_atom_forge_int(&m_forge, int32_t(ParamBase + index));
+		lv2_atom_forge_float(&m_forge, samplv1::paramValue(index));
+	}
+
+	lv2_atom_forge_pop(&m_forge, &tup_frame);
+	lv2_atom_forge_pop(&m_forge, &obj_frame);
+
+	return true;
+}
+
+#endif	// CONFIG_LV2_PORT_EVENT
+
+
 //-------------------------------------------------------------------------
 // samplv1_lv2 - LV2 desc.
 //
