@@ -1,7 +1,7 @@
 // samplv1widget_config.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2020, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -53,8 +53,8 @@ static const char *g_pszDefName = QT_TRANSLATE_NOOP("samplv1widget_config", "(de
 
 // ctor.
 samplv1widget_config::samplv1widget_config (
-	samplv1_ui *pSamplUi, QWidget *pParent, Qt::WindowFlags wflags )
-	: QDialog(pParent, wflags), p_ui(new Ui::samplv1widget_config), m_ui(*p_ui),
+	samplv1_ui *pSamplUi, QWidget *pParent )
+	: QDialog(pParent), p_ui(new Ui::samplv1widget_config), m_ui(*p_ui),
 		m_pSamplUi(pSamplUi)
 {
 	// Setup UI struct...
@@ -533,7 +533,7 @@ void samplv1widget_config::tuningScaleFileClicked (void)
 	const QString& sFilter = filters.join(";;");
 
 	QWidget *pParentWidget = nullptr;
-	QFileDialog::Options options = 0;
+	QFileDialog::Options options;
 	if (pConfig->bDontUseNativeDialogs) {
 		options |= QFileDialog::DontUseNativeDialog;
 		pParentWidget = QWidget::window();
@@ -582,7 +582,7 @@ void samplv1widget_config::tuningKeyMapFileClicked (void)
 	const QString& sFilter = filters.join(";;");
 
 	QWidget *pParentWidget = nullptr;
-	QFileDialog::Options options = 0;
+	QFileDialog::Options options;
 	if (pConfig->bDontUseNativeDialogs) {
 		options |= QFileDialog::DontUseNativeDialog;
 		pParentWidget = QWidget::window();
@@ -756,6 +756,14 @@ void samplv1widget_config::accept (void)
 		pConfig->iFrameTimeFormat = m_ui.FrameTimeFormatComboBox->currentIndex();
 		pConfig->fRandomizePercent = float(m_ui.RandomizePercentSpinBox->value());
 		int iNeedRestart = 0;
+		if (pConfig->sCustomStyleTheme != sOldCustomStyleTheme) {
+			if (pConfig->sCustomStyleTheme.isEmpty()) {
+				++iNeedRestart;
+			} else {
+				QApplication::setStyle(
+					QStyleFactory::create(pConfig->sCustomStyleTheme));
+			}
+ 		}
  		if (pConfig->sCustomColorTheme != sOldCustomColorTheme) {
 			if (pConfig->sCustomColorTheme.isEmpty()) {
 				++iNeedRestart;
@@ -768,14 +776,6 @@ void samplv1widget_config::accept (void)
 		}
 		if (pConfig->iFrameTimeFormat != iOldFrameTimeFormat)
 			++iNeedRestart;
-		if (pConfig->sCustomStyleTheme != sOldCustomStyleTheme) {
-			if (pConfig->sCustomStyleTheme.isEmpty()) {
-				++iNeedRestart;
-			} else {
-				QApplication::setStyle(
-					QStyleFactory::create(pConfig->sCustomStyleTheme));
-			}
- 		}
 		// Show restart message if needed...
  		if (iNeedRestart > 0) {
 			QMessageBox::information(this,
