@@ -1,7 +1,7 @@
 // samplv1widget_palette.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2020, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -74,6 +74,9 @@ static struct
 	{ "LinkVisited",     QPalette::LinkVisited     },
 	{ "ToolTipBase",     QPalette::ToolTipBase     },
 	{ "ToolTipText",     QPalette::ToolTipText     },
+#if QT_VERSION >= QT_VERSION_CHECK(5, 12, 0)
+	{ "PlaceholderText", QPalette::PlaceholderText },
+#endif
 	{ "NoRole",          QPalette::NoRole          },
 
 	{  nullptr,          QPalette::NoRole          }
@@ -111,9 +114,6 @@ samplv1widget_palette::samplv1widget_palette ( QWidget *parent, const QPalette&
 	m_ui.paletteView->setColumnHidden(2, true);
 	m_ui.paletteView->setColumnHidden(3, true);
 
-	QObject::connect(m_ui.nameCombo,
-		SIGNAL(activated(const QString&)),
-		SLOT(nameComboActivated(const QString&)));
 	QObject::connect(m_ui.nameCombo,
 		SIGNAL(editTextChanged(const QString&)),
 		SLOT(nameComboChanged(const QString&)));
@@ -225,18 +225,14 @@ QSettings *samplv1widget_palette::settings (void) const
 }
 
 
-void samplv1widget_palette::nameComboActivated ( const QString& name )
-{
-	setPaletteName(name);
-}
-
-
 void samplv1widget_palette::nameComboChanged ( const QString& name )
 {
-	if (m_dirtyCount > 0 || m_ui.nameCombo->findText(name) < 0)
+	if (m_dirtyCount > 0 || m_ui.nameCombo->findText(name) < 0) {
 		updateDialogButtons();
-	else
+	} else {
 		setPaletteName(name);
+		++m_dirtyTotal;
+	}
 }
 
 
@@ -512,7 +508,9 @@ bool samplv1widget_palette::namedPalette (
 	QSettings *settings, const QString& name, QPalette& pal, bool fixup )
 {
 	int result = 0;
+#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
 	uint mask = pal.resolve();
+#endif
 
 	// Custom color themes...
 	if (name == "Wonton Soup") {
@@ -573,7 +571,9 @@ bool samplv1widget_palette::namedPalette (
 		pal.setColor(QPalette::Active,   QPalette::LinkVisited, QColor(64, 128, 255));
 		pal.setColor(QPalette::Inactive, QPalette::LinkVisited, QColor(64, 128, 255));
 		pal.setColor(QPalette::Disabled, QPalette::LinkVisited, QColor(54, 76, 119));
+	#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
 		mask = 0;
+	#endif
 		++result;
 	}
 	else
@@ -635,7 +635,9 @@ bool samplv1widget_palette::namedPalette (
 		pal.setColor(QPalette::Active,   QPalette::LinkVisited, QColor(230, 100, 230));
 		pal.setColor(QPalette::Inactive, QPalette::LinkVisited, QColor(230, 100, 230));
 		pal.setColor(QPalette::Disabled, QPalette::LinkVisited, QColor(74, 34, 74));
+	#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
 		mask = 0;
+	#endif
 		++result;
 	}
 	else
@@ -653,7 +655,9 @@ bool samplv1widget_palette::namedPalette (
 				pal.setColor(QPalette::Active,   cr, QColor(clist.at(0)));
 				pal.setColor(QPalette::Inactive, cr, QColor(clist.at(1)));
 				pal.setColor(QPalette::Disabled, cr, QColor(clist.at(2)));
+			#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
 				mask &= ~(1 << int(cr));
+			#endif
 				++result;
 			}
 		}
@@ -692,7 +696,9 @@ bool samplv1widget_palette::namedPalette (
 		++result;
 	}
 
+#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
 	pal.resolve(mask);
+#endif
 	return (result > 0);
 }
 
